@@ -1,6 +1,6 @@
 ## -*- mode: Perl -*-
 ##
-## Copyright (c) 2012, 2013, 2015 The University of Utah
+## Copyright (c) 2012, 2013, 2015, 2016, 2018 The University of Utah
 ## All rights reserved.
 ##
 ## This file is distributed under the University of Illinois Open Source
@@ -16,18 +16,19 @@ use Exporter::Lite;
 use File::Spec;
 use File::Which;
 
-@EXPORT      = qw($VERBOSE $OK $STOP
+@EXPORT      = qw($DEBUG $OK $STOP $ERROR
 		  find_external_program
-		  runit 
+		  runit nprocs
                   run_clang_delta
 		  $replace_cont $matched replace_aux
 		  read_file write_file
                   );
 
-$VERBOSE = 0;
+$DEBUG = 0;
 
 $OK = 999999;
 $STOP = 111333;
+$ERROR = 223334;
 
 sub find_external_program($$) {
     my ($configured_path, $program_name) = @_;
@@ -45,16 +46,9 @@ sub find_external_program($$) {
 
 sub runit ($) {
     (my $cmd) = @_;
-    if ((system "$cmd") != 0) {
-        my $res = $? >> 8;
-        if ($res == 255) {
-            return -1;
-        }
-        else {
-            return -2;
-        }
-    }
-    return ($? >> 8);
+    my $res = system("$cmd");
+    return 0 if $res == 0;
+    return $? >> 8;
 }
 
 sub run_clang_delta ($) {
@@ -77,7 +71,7 @@ sub run_clang_delta ($) {
 # utility code to help us replace the nth occurrence of a pattern
 $replace_cont = 0;
 $matched = 0;
-sub replace_aux ($$$) { 
+sub replace_aux ($$$) {
     my ($index,$original,$replacement) = @_;
     $replace_cont++;
     $matched = 1;
@@ -92,12 +86,6 @@ sub read_file ($) {
 	$prog .= $line;
     }
     close INF;
-    if (substr($prog, 0, 1) ne " ") {
-	$prog = " $prog";
-    }
-    if (substr ($prog, -1, 1) ne " ") {
-	$prog = "$prog ";
-    }
     return $prog;
 }
 
@@ -108,4 +96,46 @@ sub write_file ($$) {
     close OUTF;
 }
 
+# attempt to find number of real cores, not hyperthreaded ones
+sub ncpus () {
+    my $OS = $^O;
+    if ($OS eq "linux") {
+	my $cores;
+	my $sockets;
+	open INF, "lscpu |";
+	while (my $line = <INF>) {
+	    chomp $line;
+	    $cores = $1 if ($line =~ /Core\(s\) per socket:\s+([0-9]+)\s*$/);
+	    $sockets = $1 if ($line =~ /Socket\(s\):\s+([0-9]+)\s*$/);
+	}
+	close INF;
+	return ($cores * $sockets) if (defined($cores) && defined($sockets));
+    }
+    if ($OS eq "darwin") {
+	my $cpus;
+	open INF, "sysctl hw |";
+	while (my $line = <INF>) {
+	    chomp $line;
+	    $cpus = $1 if ($line =~ /hw.physicalcpu: ([0-9]+)$/);
+	}
+	close INF;
+	return $cpus if defined $cpus;
+    }
+    if ($OS eq "MSWin32") {
+	# TODO
+    }
+    return 1;
+}
+
+# here we're pretty conservative about the number of parallel
+# processes to use; if the user has some big iron she can specify a
+# higher number using -n
+sub nprocs () {
+    my $cpus = ncpus();
+    die unless ($cpus >= 1);
+    return $cpus if ($cpus <= 2);
+    return $cpus - 1 if ($cpus <= 4);
+    return 4;
+}
+
 1;
